==> google-cloud-storage-2.38.0-sources.jar: com/google/cloud/storage/LazyWriteChannel.java <==
/*
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.cloud.storage;

import com.google.cloud.storage.BufferedWritableByteChannelSession.BufferedWritableByteChannel;
import java.util.function.Supplier;
import org.checkerframework.checker.nullness.qual.MonotonicNonNull;
import org.checkerframework.checker.nullness.qual.NonNull;

final class LazyWriteChannel<T> {

  private final Supplier<BufferedWritableByteChannelSession<T>> sessionSupplier;

  @MonotonicNonNull private volatile BufferedWritableByteChannelSession<T> session;
  @MonotonicNonNull private volatile BufferedWritableByteChannel channel;

  private boolean open = false;

  LazyWriteChannel(Supplier<BufferedWritableByteChannelSession<T>> sessionSupplier) {
    this.sessionSupplier = sessionSupplier;
  }

  @NonNull
  BufferedWritableByteChannel getChannel() {
    if (channel != null) {
      return channel;
    } else {
      synchronized (this) {
        if (channel == null) {
          open = true;
          channel = getSession().open();
        }
        return channel;
      }
    }
  }

  @NonNull
  BufferedWritableByteChannelSession<T> getSession() {
    if (session != null) {
      return session;
    } else {
      synchronized (this) {
        if (session == null) {
          session = sessionSupplier.get();
        }
        return session;
      }
    }
  }

  boolean isOpen() {
    return open && getChannel().isOpen();
  }
}

==> google-cloud-storage-2.57.0-sources.jar: com/google/cloud/storage/LazyWriteChannel.java <==
/*
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.cloud.storage;

import com.google.cloud.storage.BufferedWritableByteChannelSession.BufferedWritableByteChannel;
import java.util.function.Supplier;
import org.checkerframework.checker.nullness.qual.MonotonicNonNull;
import org.checkerframework.checker.nullness.qual.NonNull;

final class LazyWriteChannel<T> {

  private final Supplier<BufferedWritableByteChannelSession<T>> sessionSupplier;

  @MonotonicNonNull private volatile BufferedWritableByteChannelSession<T> session;
  @MonotonicNonNull private volatile BufferedWritableByteChannel channel;

  private boolean open = false;

  LazyWriteChannel(Supplier<BufferedWritableByteChannelSession<T>> sessionSupplier) {
    this.sessionSupplier = sessionSupplier;
  }

  @NonNull BufferedWritableByteChannel getChannel() {
    if (channel != null) {
      return channel;
    } else {
      synchronized (this) {
        if (channel == null) {
          open = true;
          channel = getSession().open();
        }
        return channel;
      }
    }
  }

  @NonNull BufferedWritableByteChannelSession<T> getSession() {
    if (session != null) {
      return session;
    } else {
      synchronized (this) {
        if (session == null) {
          session = sessionSupplier.get();
        }
        return session;
      }
    }
  }

  boolean isOpen() {
    return open && getChannel().isOpen();
  }
}
